/**
 * Program SVGChemistry.java
 * @author Marlon Pierce
 *
 * Sets up the chemistry data I got from Paul.
 */

import java.awt.*;
import java.awt.geom.*;
import java.io.*;
import javax.swing.*;
import java.awt.image.*;

//Change the following to the new package names when I make them.
//import org.apache.batik.apps.svgviewer.Application;
//import org.apache.batik.apps.svgviewer.ViewerFrame;
import org.apache.batik.util.awt.svg.SVGGraphics2D;
import org.apache.batik.dom.svg.SVGOMDocument;


public class SVGChemistry extends WhiteBoardComp {
    protected Application application;
    protected Main main;
    protected ViewerFrame vf;
    protected GraphicsManager gm;
    protected Color color;
    protected SVGGraphics2D g2d;

    double [][] xydata;
    double xymin,xymax;
    
    //This needs to be collected from some meta data file.
    int xdim=12,ydim=12;
    int xpart=1,ypart=1;

    int frameXDim,frameYDim;

    public SVGChemistry(Main main) {
	super();
	this.main=main;
	xydata=new double[xdim][ydim];
   }
    
    /**
     * Initialize local copies of other system components.  Note that
     * we do this here and not in the constructor because their 
     * initialization completes after this particular WhiteBoardComp object.
     */
    public void init() {
	vf=main.getViewerFrame();
//  	frameXDim=vf.graphcanvas.getWidth();
//  	frameYDim=vf.graphcanvas.getHeight();
  	frameXDim=vf.thecanvas.getWidth();
  	frameYDim=vf.thecanvas.getHeight();
//  	gm=main.getGraphicsManager();
//  	gm.test();
    }

    public WhiteBoardComp getShape() {
	return this;
    }

    /**
     * This loads the data from the file.  This is file specific.
     */
    public void loadDataFile(String fileName){
	System.out.println("Loading data...");
	try {
	    BufferedReader fileReader=
		new BufferedReader(new FileReader(fileName));
	    String s;

	    //Read data from file.
	    for (int i=0;i<xdim;i++){
		for (int j=0;j<ydim;j++) {
		    s=fileReader.readLine();
		    xydata[i][j]=Double.parseDouble(s);
		}
	    }
	    
	    //Determine maximum value.
  	    xymin=xydata[0][0];
  	    xymax=xydata[0][0];
  	    for (int i=0;i<xdim;i++){
  		for (int j=0;j<ydim;j++) {
  		    if(xydata[i][j]>xymax)
  			xymax=xydata[i][j];
  		    if(xydata[i][j]<xymin)
  			xymin=xydata[i][j];
  		}
  	    }
	    
	    
	}
	catch(IOException ex) {
	    System.err.println("Couldn't open the specified data file");
	    ex.printStackTrace();
	}
    }
    
    /**
     * Create the graphics object for the shape.  
     */
    
    //Some hacked up ugly code that I should improve on.
    public void drawShape(SVGGraphics2D g2d, int ipart,int jpart) {    	
	this.g2d=g2d;
//  	BufferedImage image=new BufferedImage(vf.graphcanvas.getWidth(),
//  					      vf.graphcanvas.getHeight(),
//  					      BufferedImage.TYPE_INT_ARGB);

  	BufferedImage image=new BufferedImage(vf.thecanvas.getWidth(),
  					      vf.thecanvas.getHeight(),
  					      BufferedImage.TYPE_INT_ARGB);
	
	Graphics2D gImage=image.createGraphics();

	//Make this transparent
//  	int rule=AlphaComposite.SRC_OVER;
//  	float alpha=0.0f;   //Make it transparent.
	
	int red,green,blue,index;
	int vertical_boxes=xdim;
	int horiz_boxes=ydim;
	double xyval;
	
	//The joys of integer division.
	int dx=(frameXDim)/horiz_boxes/xpart;
	int dy=(frameYDim)/vertical_boxes/ypart;
	
	System.out.println("About to draw shape...");
	long start;
	long finish;
	start=System.currentTimeMillis();
	setYPosition(0);

	System.out.println("Test:"+ipart+" "+jpart);
	for(int i=0;i<horiz_boxes;i++) {
	    for(int j=0;j<vertical_boxes;j++) {
//  		int ishift=ipart*horiz_boxes+i;
//  		int jshift=(jpart+1)*vertical_boxes-1-j;
  		int ishift=i;
  		int jshift=vertical_boxes-1-j;
		int xstart=ishift*dx;
		int ystart=jshift*dy;
		xyval=xydata[ishift][jshift];
		index=(int)((2.*256.-1.)*(xyval-xymin)/(xymax-xymin));
		if(index<256){
		    blue=255-index;
		    green=index;
		    red=0;
		}
		else if(index==256){
		    blue=0;
		    green=255;
		    red=0;
		}
		else if((index>256)&&(index<=511)){
		    blue=0;
		    green=511-index;
		    red=index-256;
		}
		//Something went wrong, so mark as white.
		else {
		    System.out.println("Invalid index: "+index);
		    blue=255;
		    green=255;
		    red=255;
		}
		float redf=(float)red/(float)255;
		float greenf=(float)green/(float)255;
		float bluef=(float)blue/(float)255;
		
		g2d.setColor(new Color(redf,greenf,bluef));
		g2d.fillRect(xstart,ystart+getYPosition(),dx,dy);
	    }
	}

	finish=System.currentTimeMillis();
	System.out.println("Shape drawn.");
	System.out.println(finish-start);
	
	//Now create the new document and load the shape.
  	SVGOMDocument chemdoc=writeSVGDocToMemory(g2d);
	//	SVGOMDocument chemdoc1=
	// updateBaseDoc(chemdoc,vf.getGraphDoc());
	//	vf.appendFragToDoc(vf.graphcanvas,chemdoc1);

	SVGOMDocument chemdoc1=updateBaseDoc(chemdoc,vf.getTheDoc());
	vf.appendFragToDoc(vf.thecanvas,chemdoc1);
    }
    
    /**
     * Provide access to the X and Y partitioning sizes.  The data may
     * be partitioned for quicker loading and display.
     */
    public int getXPart() { return xpart; }
    public int getYPart() { return ypart; }
}



