import ice.storm.*;
import java.awt.*;
import java.awt.event.*;
import java.beans.*;

import org.w3c.dom.*;
import org.w3c.dom.html.*;
import org.w3c.dom.events.*;

// HTML4 pilot, doc, DOM impl
import ice.pilots.html4.*;

import java.net.URL;

import java.util.Hashtable;
import java.io.StringReader;

public class MyCallback implements StormCallback {

	private Hashtable windows = new Hashtable();
	private StormBase base;
  boolean controlFlag = false;

	public void init(StormBase base) {
		this.base=base;
	}

	public Container createTopLevelContainer(Viewport viewport) {
		BrowserFrame f = new BrowserFrame(base,viewport);
                f.controlFlag = controlFlag;
                windows.put(viewport.getId(),f);
		f.setVisible(true);
		return f.getPanel();
	}

	public void disposeTopLevelContainer(Viewport viewport) {
		BrowserFrame f = (BrowserFrame)windows.get(viewport.getId());
		if (f!=null) {
			windows.remove(viewport.getId());
			f.setVisible(false);
			f.dispose();
		}

		if (windows.isEmpty()) {
			System.exit(0);
		}
	}

	public void message(Viewport vp, String msg) {
		Frame f = findFrame(vp);
		if (f == null) return;

		MessageBox b = new MessageBox(f,msg,true);
		packAndShow(b,f);
	}

	public boolean confirm(Viewport vp, String msg) {
		Frame f = findFrame(vp);
		if (f == null) return false;

		ConfirmBox b = new ConfirmBox(f,msg,true);
		packAndShow(b,f);

		return b.response;
	}

	public String prompt(Viewport vp, String msg, String defVal) {

		Frame f = findFrame(vp);
		if (f == null) return null;

		PromptBox b = new PromptBox(f,msg,true,defVal);
		packAndShow(b,f);

		return b.responseString;
	}

	private Frame findFrame(Viewport vp) {
		Component x = (Component)vp.getContainer();
		if (x == null) return null;

		while(x.getParent() != null && !(x instanceof Frame)) {
			x = x.getParent();
		}
		if (!(x instanceof Frame)) return null;

		return (Frame)x;
	}

	private void packAndShow(Dialog d, Frame f) {
		d.pack();
		if (f!=null) {
			Point p = f.getLocation();
			Dimension fd = f.getSize();
			Dimension dd = d.getSize();
			int x = p.x+(fd.width-dd.width)/2;
			if (x<0) x=0;
			int y = p.y+(fd.height-dd.height)/2;
			if (y<0) y=0;
			d.setLocation(x,y);
		}
		d.show();
		d.toFront();
	}
}

class MessageBox extends Dialog implements ActionListener {

    public MessageBox(Frame topLevel, String titleIn, boolean modal) {
        super(topLevel,"ICE Browser",modal);

        Panel p = new Panel();
        p.add(new Label(titleIn));
        add(p,"North");
        Button b = new Button("OK");
        b.addActionListener(this);
        add(b,"South");
    }
    public void actionPerformed(ActionEvent e) {
	setVisible(false); dispose();
    }
}

class ConfirmBox extends Dialog implements ActionListener {

    private Button okButton=new Button("OK");
    private Button cancelButton=new Button("Cancel");
    boolean response=false;


    public ConfirmBox(Frame topLevel, String messageIn, boolean modal) {
        super(topLevel,"ICE Browser",modal);

        Panel p = new Panel();
        p.add(okButton);
        p.add(cancelButton);

        okButton.addActionListener(this);
        cancelButton.addActionListener(this);

        add(p,"South");
        add(new Label(messageIn),"North");
    }

    public void actionPerformed(ActionEvent e) {
        response=(e.getSource()==okButton);
        setVisible(false);
        dispose();
    }
}

class PromptBox extends Dialog implements ActionListener {

    private Button okButton=new Button("OK");
    private Button cancelButton=new Button("Cancel");
    private TextField theText=null;
    String responseString=null;

    public PromptBox(Frame topLevel, String messageIn, boolean modal,String defaultValueIn) {
        super(topLevel,"ICE Browser",modal);

        theText=new TextField(defaultValueIn,30);
        theText.addActionListener(this);

        Panel p = new Panel();
        p.setLayout(new GridLayout(2,1));
        p.add(new Label(messageIn));
        p.add(theText);
        add(p,"North");

        okButton.addActionListener(this);
        cancelButton.addActionListener(this);

        p = new Panel();
        p.add(okButton);
        p.add(cancelButton);
        add(p,"South");
    }

    public void actionPerformed(ActionEvent e) {
        Object src = e.getSource();
        if (src==okButton||src==theText) {
            responseString=theText.getText();
        }
        setVisible(false);
        dispose();
    }
}









