/**
 * Program: WhiteBoardComp.java
 * @author Marlon Pierce
 *
 * Defines the abstract parent class for all of the shapes I will draw on
 * the whiteboard.  Also defines the method writeSVGDocToMemory that all will
 * need to employ to translate the images they generate into internal SVG.
 */
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.util.*;
import java.io.*;
import javax.swing.*;
import org.apache.batik.util.awt.svg.SVGGraphics2D;
import org.apache.batik.dom.svg.SVGOMDocument;
import org.apache.batik.dom.svg.SVGDocumentFactory;
import org.xml.sax.InputSource;
import org.w3c.dom.DocumentFragment;
import org.apache.batik.dom.GenericDocumentFragment;
import org.w3c.dom.*;
import org.apache.batik.util.awt.svg.*;
import org.apache.batik.util.SVGConstants;

public abstract class WhiteBoardComp {
    protected int yposition;
    public WhiteBoardComp() {
    }
    public abstract WhiteBoardComp getShape();
    public abstract void loadDataFile(String filename);
    public abstract void drawShape(SVGGraphics2D g2d,int xpart,int ypart);
    public abstract void init();
    public abstract int getXPart();
    public abstract int getYPart();
    
    public String toString(){
	return getClass().getName();
    }
    
    /**
     * The following two methods are accessors for yposition, 
     * which is the current last line of the image.
     */
    public int getYPosition() { return yposition; }
    public void setYPosition(int yp) { yposition=yp; }
    

    /**
     * This method moves the newly created document in the SVGGraphics2D 
     * instance
     * to a memory buffer, and then uses this buffer as the source for a new
     * SVGOMDocument that can be passed directly to the ViewerFrame.
     */
    public DocumentFragment writeSVGFragToMemory(SVGGraphics2D svgGenerator){

	//Write the new SVG file to disk.
	long start;
	long finish;
	start=System.currentTimeMillis();
	
	BufferedWriter bufOut;
	BufferedReader bufRead;
	CharArrayWriter out;
	CharArrayReader charReader;
	
	SVGDocumentFactory df;
	SVGOMDocument newdoc=null;
	DocumentFragment frag=null;
	
	//Writes the SVG file to a character array, then reads it
	//back in to a new document.  Should also be able to do this
	//with PipedReader/Writers but couldn't get that to work.  The
	//URI "tmp.svg" is just a placebo and does not actually have to
	//exist.  
	try { 

	    //---------------------------------------------------------
	    //The following is part of the original working code.
	    out=new CharArrayWriter();
	    bufOut=new BufferedWriter(out);
	    svgGenerator.stream(bufOut,true);
	    char[] charArray=out.toCharArray();
	    charReader=new CharArrayReader(charArray);	    
	    bufRead=new BufferedReader(charReader);
	    df=new SVGDocumentFactory(getXMLParserClassName());
	    newdoc=
		df.createDocument("file:/tmp.svg",new InputSource(bufRead));

  	    //Possibly frees up memory.
  	    out.flush();
  	    out.close();
  	    bufOut.flush();
  	    bufOut.close();
	    //--------------------------------------------------------

	    //--------------------------------------------------------
	    //Try putting everything into a fragment.
//      	    DOMTreeManager dtm=svgGenerator.getDOMTreeManager();
//      	    Element svgRoot=dtm.getRoot();

    	    Element svgRoot=newdoc.getDocumentElement();

//    	    frag=svgRoot.getOwnerDocument().createDocumentFragment();
	    frag=newdoc.createDocumentFragment();
  	    frag.appendChild(svgRoot);

  	    Writer stdout=new OutputStreamWriter(System.out,"UTF-8");
  	    System.out.println("frag test 1");
  	    XmlWriter.writeXml(frag,stdout);
  	    stdout.flush();
	}
	catch(Exception ex) {
	    ex.printStackTrace();
	}
	
	System.out.println("File has been written");
	finish=System.currentTimeMillis();
	System.out.println(finish-start);
	
//  	return newdoc;
	return frag;
    }
    
    /**
     * This is the original method.
     */
    public SVGOMDocument writeSVGDocToMemory(SVGGraphics2D svgGenerator){
	//Write the new SVG file to disk.
	long start;
	long finish;
	start=System.currentTimeMillis();
	
	BufferedWriter bufOut;
	BufferedReader bufRead;
	CharArrayWriter out;
	CharArrayReader charReader;
	
	SVGDocumentFactory df;
	SVGOMDocument newdoc=null;
	DocumentFragment frag=null;
	
	//Writes the SVG file to a character array, then reads it
	//back in to a new document.  Should also be able to do this
	//with PipedReader/Writers but couldn't get that to work.  The
	//URI "tmp.svg" is just a placebo and does not actually have to
	//exist.  
	try { 
  	    //Print also to screen
  	    Writer stdout=new OutputStreamWriter(System.out,"UTF-8");
//  	    svgGenerator.stream(stdout,true);
	    
	    //---------------------------------------------------------
	    //The following is part of the original working code.
	    out=new CharArrayWriter();
	    bufOut=new BufferedWriter(out);
	    svgGenerator.stream(bufOut,true);
	    char[] charArray=out.toCharArray();
	    charReader=new CharArrayReader(charArray);	    
	    bufRead=new BufferedReader(charReader);
	    df=new SVGDocumentFactory(getXMLParserClassName());
	    newdoc=
		df.createDocument("file:/tmp.svg",new InputSource(bufRead));

  	    //Possibly frees up memory.
  	    out.flush();
  	    out.close();
  	    bufOut.flush();
  	    bufOut.close();

//  	    System.out.println("Writer test");
//  	    XmlWriter.writeXml(newdoc,stdout);
//  	    stdout.flush();

	    //--------------------------------------------------------
	    
	}
	catch(Exception ex) {
	    ex.printStackTrace();
	}
	
	System.out.println("File has been written");
	finish=System.currentTimeMillis();
	System.out.println(finish-start);
	
  	return newdoc;
    }
    
    //Should obtain this from Main.
    protected String getXMLParserClassName() {
	return "org.apache.crimson.parser.XMLReaderImpl";
    }
    /**
     * Add the recently created <g> to the working document.
     */
    public SVGOMDocument updateBaseDoc(SVGOMDocument frag,
				       SVGOMDocument base) {
	SVGOMDocument svgomdoc=base;
	org.w3c.dom.Node root=svgomdoc.getDocumentElement();
	org.w3c.dom.Node savenode=null,oldchild=null;
	org.w3c.dom.NodeList nl=frag.getChildNodes(),nl2,nl3,nl4,nl5;
	org.w3c.dom.Element tmpelem;
	
	//Search the svgomdoc object for the svg 
	//tag and save it.
  	nl4=svgomdoc.getChildNodes();
  	for(int i=0;i<nl4.getLength();i++) {
	    if(nl4.item(i).getNodeName().equals(SVGConstants.TAG_SVG)) {
		savenode=(org.w3c.dom.Element)nl4.item(i);
		oldchild=savenode.getFirstChild();
		while(!oldchild.getNodeName().equals(SVGConstants.TAG_G))
		    oldchild=oldchild.getNextSibling();
	    }
  	}
	
	//Copy the graphics object tag contents to a new
	//<g> and append this to the local document.
	org.w3c.dom.Element g_elem=svgomdoc.createElement(SVGConstants.TAG_G);
	for(int i=0;i<nl.getLength();i++) {
	    nl2=nl.item(i).getChildNodes();
	    for(int j=0;j<nl2.getLength();j++) {
		if(nl2.item(j).getNodeName().equals(SVGConstants.TAG_G)) {
		    nl3=nl2.item(j).getChildNodes();
		    for(int k=0;k<nl3.getLength();k++) {
			nl4=nl3.item(k).getChildNodes();
			org.w3c.dom.Element g_child=
			    svgomdoc.createElement(nl3.item(k).getNodeName());
			tmpelem=(org.w3c.dom.Element)nl3.item(k);
			if(tmpelem.hasAttribute("style")) {
			    g_child.setAttribute("style",
						 tmpelem.getAttribute("style"));
			}

			for(int l=0;l<nl4.getLength();l++) {
			    tmpelem=(org.w3c.dom.Element)nl4.item(l);
			    org.w3c.dom.Element g_grandchild=
				svgomdoc.createElement(nl4.item(l).getNodeName());
			    NamedNodeMap nnm=tmpelem.getAttributes();
			    for(int m=0;m<nnm.getLength();m++) {
				g_grandchild.setAttribute(nnm.item(m).getNodeName(),nnm.item(m).getNodeValue());
			    }
			    for(Node ch=tmpelem.getFirstChild();
				ch!=null;ch=ch.getNextSibling()) {
				if(ch.getNodeType()==Node.TEXT_NODE) {
				    String textvalue=ch.getNodeValue();
				    g_grandchild.appendChild(svgomdoc.createTextNode(textvalue));
				}
				
			    }
			    g_child.appendChild(g_grandchild);
			}
			g_elem.appendChild(g_child);
		    }
		}
	    }
	}

  	savenode.appendChild(g_elem);
	
	//Now create a new doc from scratch using the modified 
	//nodes above.
	
	SVGOMDocument newdoc=null;
	try {
	    CharArrayWriter out=new CharArrayWriter();
	    BufferedWriter bufOut=new BufferedWriter(out);
	    
      	    XmlWriter.writeXml(svgomdoc,out);
	    
	    char[] charArray=out.toCharArray();
	    CharArrayReader charReader=new CharArrayReader(charArray);	    
	    BufferedReader bufRead=new BufferedReader(charReader);
	    SVGDocumentFactory df=
		new SVGDocumentFactory(getXMLParserClassName());
	    newdoc=
		df.createDocument("file:/tmp.svg",new InputSource(bufRead));
	    
	    //Possibly frees up memory.
	    out.flush();
	    out.close();
	    bufOut.flush();
	    bufOut.close();
	}
	catch(Exception ex) { ex.printStackTrace(); }
	
	return newdoc;
    }
}

