/*
 * vt320 -- a DEC VT320 Terminal emulation
 * --
 * $Id: vt320.java,v 1.56 1998/03/07 23:47:03 marcus Exp $
 *
 * This file is part of "The Java Telnet Applet".
 *
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * "The Java Telnet Applet" is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

package display;

import java.awt.Scrollbar;
import java.awt.Event;
import java.awt.Dimension;
import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.util.Vector;

import java.applet.Applet;

/**
 * A DEC VT320 Terminal Emulation (includes VT100/220 and ANSI).
 *
 * The terminal emulation accesses the applet parameters to configure itself.
 * The following parameters may be set. Default values are written in
 * <I>italics</I> and other possible values are <B>bold</B>.
 * <DL>
 *  <DT><TT>&lt;PARAM NAME="Fx" VALUE="<I>functionkeytext</I>"&gt</TT>
 *  <DD>Sets the string sent when the function key Fx (x between 1 und 20)
 *	is pressed.
 *  <DT><TT>&lt;PARAM NAME="VTcolumns" VALUE="<I>80</I>"&gt</TT>
 *  <DD>Sets the columns of the terminal initially. If the parameter
 *      VTresize is set to <B>screen</B> this may change, else it is fixed.
 *  <DT><TT>&lt;PARAM NAME="VTrows" VALUE="<I>24</I>"&gt</TT>
 *  <DD>Sets the rows of the terminal initially. If the parameter
 *      value of VTresize <B>screen</B> this may change!
 *  <DT><TT>&lt;PARAM NAME="VTfont" VALUE="<I>Courier</I>"&gt</TT>
 *  <DD>Sets the font to be used for the terminal. It is recommended to
 *      use <I>Courier</I> or at least a fixed width font.
 *  <DT><TT>&lt;PARAM NAME="VTfontsize" VALUE="<I>14</I>"&gt</TT>
 *  <DD>Sets the font size for the terminal. If the parameter
 *      value of VTresize is set to <B>font</B> this may change!
 *  <DT><TT>&lt;PARAM NAME="VTresize" VALUE="<I>font</I>"&gt</TT>
 *  <DD>This parameter determines what the terminal should do if the window
 *      is resized. The default setting <I><B>font</B></I> will result in
 *      resizing the font until is matches the window best. Other possible
 *      values are <B>none</B> or <B>screen</B>. <B>none</B> will let nothing
 *      happen and <B>screen</B> will let the display try to change the
 *      amount of rows and columns to match the window best.
 *  <DT><TT>&lt;PARAM NAME="VTscrollbar" VALUE="<I>false</I>"&gt</TT>
 *  <DD>Setting this parameter to <B>true</B> will add a scrollbar west to
 *      the terminal. Other possible values include <B>left</B> to put the
 *      scrollbar on the left side of the terminal and <B>right</B> to put it
 *      explicitely to the right side.
 *  <DT><TT>&lt;PARAM NAME="VTid" VALUE="<I>vt320</I>"&gt</TT>
 *  <DD>This parameter will override the terminal id <I>vt320</I>. It may
 *      be used to determine special terminal abilities of VT Terminals.
 *  <DT><TT>&lt;PARAM NAME="VTbuffer" VALUE="<I>xx</I>"&gt</TT>
 *  <DD>Initially this parameter is the same as the VTrows parameter. It
 *      cannot be less than the amount of rows on the display. It determines
 *      the available scrollback buffer.
 *  <DT><TT>&lt;PARAM NAME="VTcharset" VALUE="<I>none</I>"&gt</TT>
 *  <DD>Setting this parameter to <B>ibm</B> will enable mapping of ibm
 *      characters (as used in PC BBS systems) to UNICODE characters. Note
 *      that those special characters probably won't show on UNIX systems
 *      due to lack in X11 UNICODE support.
 *  <DT><TT>&lt;PARAM NAME="VTvms" VALUE="<I>false</I>"&gt</TT>
 *  <DD>Setting this parameter to <B>true</B> will change the Backspace key
 *      into a delete key, cause the numeric keypad keys to emit VT100
 *      codes when Ctrl is pressed, and make other VMS-important keyboard
 *      definitions.
 *  <DT><TT>&lt;PARAM NAME="F<I>nr</I>" VALUE="<I>string</I>"&gt</TT>
 *  <DD>Function keys from <I>F1</I> to <I>F20</I> are programmable. You can
 *      install any possible string including special characters, such as
 *      <TABLE BORDER>
 *      <TR><TD><TT>\e</TT></TD><TD>Escape</TD>
 *      <TR><TD><TT>\b</TT></TD><TD>Backspace</TD>
 *      <TR><TD><TT>\n</TT></TD><TD>Newline</TD>
 *      <TR><TD><TT>\r</TT></TD><TD>Return</TD>
 *      </TABLE>
 *  <DT><TT>&lt;PARAM NAME="CF<I>nr</I>" VALUE="<I>string</I>"&gt</TT>
 *  <DD>Function keys (with the Control-key pressed) from <I>CF1</I> to <I>CF20</I> are programmable too.
 *  <DT><TT>&lt;PARAM NAME="SF<I>nr</I>" VALUE="<I>string</I>"&gt</TT>
 *  <DD>Function keys (with the Shift-key pressed) from <I>SF1</I> to <I>SF20</I> are programmable too.
 *  <DT><TT>&lt;PARAM NAME="AF<I>nr</I>" VALUE="<I>string</I>"&gt</TT>
 *  <DD>Function keys (with the Alt-key pressed) from <I>AF1</I> to <I>AF20</I> are programmable too.
 * </DL>
 * @version $Id: vt320.java,v 1.56 1998/03/07 23:47:03 marcus Exp $
 * @author  Matthias L. Jugel, Marcus Mei?ner
 */
public class vt320 extends Terminal implements TerminalHost
{
  /**
   * Return the version of the terminal emulation and its display.
   */
 // public String toString() { return "$Id: vt320.java,v 1.56 1998/03/07 23:47:03 marcus Exp $ "+display.version; }

  // the input handler takes the keyboard input from us.
  private TerminalHost host = this;

  // due to a bug with Windows we need a keypress cache
  private int pressedKey = ' ';
  private long pressedWhen = ' ';

  
  private static int debug = 0;
  private String terminalID = "vt320";

  // X - COLUMNS, Y - ROWS
  int  R,C;
  int  Sc,Sr,Sa;
  int  attributes  = 0;
  int  insertmode  = 0;
  int  statusmode = 0;
  int  vt52mode  = 0;
  int  normalcursor  = 0;
  boolean originmode = false;
  boolean sendcrlf = true;

  private boolean  useibmcharset = false;

  private  static  int  lastwaslf = 0;
  private static  int i;
  private final static char ESC = 27;
  private final static char IND = 132;
  private final static char NEL = 133;
  private final static char RI  = 141;
  private final static char HTS = 136;
  private final static char DCS = 144;
  private final static char CSI = 155;
  private final static char OSC = 157;
  private final static int TSTATE_DATA  = 0;
  private final static int TSTATE_ESC  = 1; /* ESC */
  private final static int TSTATE_CSI  = 2; /* ESC [ */
  private final static int TSTATE_DCS  = 3; /* ESC P */
  private final static int TSTATE_DCEQ  = 4; /* ESC [? */
  private final static int TSTATE_ESCSQUARE= 5; /* ESC # */
  private final static int TSTATE_OSC= 6;       /* ESC ] */
  private final static int TSTATE_SETG0= 7;     /* ESC (? */
  private final static int TSTATE_SETG1= 8;     /* ESC )? */
  private final static int TSTATE_SETG2= 9;     /* ESC *? */
  private final static int TSTATE_SETG3= 10;    /* ESC +? */
  private final static int TSTATE_CSI_DOLLAR  = 11; /* ESC [ Pn $ */

  /* The graphics charsets
   * B - default ASCII
   * A - default UK
   * 0 - DEC SPECIAL
   * < - User defined
   * ....
   */
  private static char gx[] = {
  'B',      // g0
  '0',      // g1
  'A',      // g2
  '<',      // g3
  };
  private static char gr = 1;  // default GR to G1
  private static char gl = 0;  // default GL to G0

  // array to store DEC Special -> Unicode mapping
  //  Unicode   DEC  Unicode name    (DEC name)
  private static char DECSPECIAL[] = {
    '\u0040', //5f blank
    '\u2666', //60 black diamond
    '\u2592', //61 grey square
    '\u2409', //62 Horizontal tab  (ht) pict. for control
    '\u240c', //63 Form Feed       (ff) pict. for control
    '\u240d', //64 Carriage Return (cr) pict. for control
    '\u240a', //65 Line Feed       (lf) pict. for control
    '\u00ba', //66 Masculine ordinal indicator
    '\u00b1', //67 Plus or minus sign
    '\u2424', //68 New Line        (nl) pict. for control
    '\u240b', //69 Vertical Tab    (vt) pict. for control
    '\u2518', //6a Forms light up   and left
    '\u2510', //6b Forms light down and left
    '\u250c', //6c Forms light down and right
    '\u2514', //6d Forms light up   and right
    '\u253c', //6e Forms light vertical and horizontal
    '\u2594', //6f Upper 1/8 block                        (Scan 1)
    '\u2580', //70 Upper 1/2 block                        (Scan 3)
    '\u2500', //71 Forms light horizontal or ?em dash?    (Scan 5)
    '\u25ac', //72 \u25ac black rect. or \u2582 lower 1/4 (Scan 7)
    '\u005f', //73 \u005f underscore  or \u2581 lower 1/8 (Scan 9)
    '\u251c', //74 Forms light vertical and right
    '\u2524', //75 Forms light vertical and left
    '\u2534', //76 Forms light up   and horizontal
    '\u252c', //77 Forms light down and horizontal
    '\u2502', //78 vertical bar
    '\u2264', //79 less than or equal
    '\u2265', //7a greater than or equal
    '\u00b6', //7b paragraph
    '\u2260', //7c not equal
    '\u00a3', //7d Pound Sign (british)
    '\u00b7'  //7e Middle Dot
  };

  private final static int KEYUP  = Event.UP % 1000;
  private final static int KEYDOWN  = Event.DOWN % 1000;
  private final static int KEYLEFT  = Event.LEFT % 1000;
  private final static int KEYRIGHT  = Event.RIGHT % 1000;
  private final static int KEYF1  = Event.F1 % 1000;
  private final static int KEYF2  = Event.F2 % 1000;
  private final static int KEYF3  = Event.F3 % 1000;
  private final static int KEYF4  = Event.F4 % 1000;
  private final static int KEYF5  = Event.F5 % 1000;
  private final static int KEYF6  = Event.F6 % 1000;
  private final static int KEYF7  = Event.F7 % 1000;
  private final static int KEYF8  = Event.F8 % 1000;
  private final static int KEYF9  = Event.F9 % 1000;
  private final static int KEYF10  = Event.F10 % 1000;
  private final static int KEYF11  = Event.F11 % 1000;
  private final static int KEYF12  = Event.F12 % 1000;
  private final static int KEYPGDN   = Event.PGDN % 1000;
  private final static int KEYPGUP   = Event.PGUP % 1000;

  private final static int KEYHOME  = Event.HOME % 1000;
  private final static int KEYEND  = Event.END % 1000;

  public static final int KEYPRINT_SCREEN  = 20;
  public static final int KEYSCROLL_LOCK    = 21;
  public static final int KEYCAPS_LOCK    = 22;
  public static final int KEYNUM_LOCK    = 23;
  public static final int KEYPAUSE    = 24;
  public static final int KEYINSERT    = 25;

    /**
     * The Insert key.
     */
    public static final int INSERT    = 1025;

  /**
   * Strings to send on function key presseic
   */
  private String FunctionKey[];
  private String FunctionKeyShift[];
  private String FunctionKeyCtrl[];
  private String FunctionKeyAlt[];
  private String KeyUp;
  private String KeyDown;
  private String KeyLeft;
  private String KeyRight;
  private String KeyBacktab;
  private String KeyTab;

  private String KP0;
  private String KP1;
  private String KP2;
  private String KP3;
  private String KP4;
  private String KP5;
  private String KP6;
  private String KP7;
  private String KP8;
  private String KP9;
  private String KPMinus;
  private String KPComma;
  private String KPPeriod;
  private String KPEnter;
  private String PF1;
  private String PF2;
  private String PF3;
  private String PF4;
  private String Help;
  private String Do;
  private String Find;
  private String Insert;
  private String Remove;
  private String Select;
  private String PrevScn;
  private String NextScn;


  private String osc,dcs;  /* to memorize OSC & DCS control sequence */

  private int term_state = TSTATE_DATA;
  private boolean vms = false;
  private byte[]  Tabs;
  private int[]  DCEvars = new int [10];
  private  int  DCEvar;

  /* operation system we run on, Scrollbar hack */
  private String osn = System.getProperty("os.name");

  public String[][] getParameterInfo() {
    String pinfo[][] = {
    {"VTcolumns",  "Integer",   "Columns of the terminal"},
    {"VTrows",     "Integer",   "Rows of the terminal"},
    {"VTfont",     "String",    "Terminal font (default is Courier)"},
    {"VTfontsize", "Integer",   "Font size"},
    {"VTbuffer",   "Integer",   "Scrollback buffer size"},
    {"VTscrollbar","Boolean",   "Enable or disable scrollbar"},
    {"VTresize",   "String",    "One of none, font, screen"},
    {"VTid",       "String",    "Terminal id, standard is VT320"},
    {"VTcharset",  "String",    "Charset used"},
    {"VTvms",      "Boolean",   "Enable or disable VMS key mappings"},
    {"F1 - F20",   "String",    "Programmable Function Keys"},
    {"SF1 - SF20",   "String",    "Programmable Shift-ed Function Keys"},
    {"CF1 - CF20",   "String",    "Programmable Control-ed Function Keys"},
    {"AF1 - AF20",   "String",    "Programmable Alt-ed Function Keys"},
    };
    return pinfo;
  }

  static String unEscape(String tmp) {
      int idx = 0, oldidx = 0;
      String cmd;

      cmd = "";
      while((idx = tmp.indexOf('\\', oldidx)) >= 0 &&
            ++idx <= tmp.length()) {
        cmd += tmp.substring(oldidx, idx-1);
        if(idx == tmp.length()) return cmd;
        switch(tmp.charAt(idx)) {
        case 'b': cmd += "\b"; break;
        case 'e': cmd += "\u001b"; break;
        case 'n': cmd += "\n"; break;
        case 'r': cmd += "\r"; break;
        case 't': cmd += "\t"; break;
        case 'v': cmd += "\u000b"; break;
        case 'a': cmd += "\u0012"; break;
        default : cmd += tmp.substring(idx, ++idx);break;
	}
	oldidx = ++idx;
      }
      if(oldidx <= tmp.length()) cmd += tmp.substring(oldidx);
      return cmd;
  }

  /**
   * Initialize terminal.
   * @param parent the applet parent where to get parameters from
   * @see display.Terminal
   */
  

  /*private void InitTerminalVars() {
    int nw = 32;

    if (nw<132) nw=132; //catch possible later 132/80 resizes
    Tabs = new byte[nw];
    for (int i=0;i<nw;i+=8) {
      Tabs[i]=1;
    }

    PF1  = "\u001bOP";
    PF2  = "\u001bOQ";
    PF3  = "\u001bOR";
    PF4  = "\u001bOS";

    Find = "\u001b[1~";
    Insert = "\u001b[2~";
    Remove = "\u001b[3~";
    Select = "\u001b[4~";
    PrevScn = "\u001b[5~";
    NextScn = "\u001b[6~";

    Help = "\u001b[28~";
    Do = "\u001b[29~";

    FunctionKey = new String[21];
    FunctionKey[0]= "";
    FunctionKey[1]= PF1;
    FunctionKey[2]= PF2;
    FunctionKey[3]= PF3;
    FunctionKey[4]= PF4;
    /* following are defined differently for vt220 / vt132 ... */
    /*FunctionKey[5]= "\u001b[15~";
    FunctionKey[6]= "\u001b[17~";
    FunctionKey[7]= "\u001b[18~";
    FunctionKey[8]= "\u001b[19~";
    FunctionKey[9]= "\u001b[20~";
    FunctionKey[10]= "\u001b[21~";
    FunctionKey[11]= "\u001b[23~";
    FunctionKey[12]= "\u001b[24~";
    FunctionKey[13]= "\u001b[25~";
    FunctionKey[14]= "\u001b[26~";
    FunctionKey[15]= Help;
    FunctionKey[16]= Do;
    FunctionKey[17]= "\u001b[31~";
    FunctionKey[18]= "\u001b[32~";
    FunctionKey[19]= "\u001b[33~";
    FunctionKey[20]= "\u001b[34~";

    FunctionKeyShift = new String[21];
    FunctionKeyAlt = new String[21];
    FunctionKeyCtrl = new String[21];

    for (int i=0;i<20;i++)
    {
       FunctionKeyShift[i]="";
       FunctionKeyAlt[i]="";
       FunctionKeyCtrl[i]="";
    }
    FunctionKeyShift[15] = Find;
    FunctionKeyShift[16] = Select;

    KeyTab   = "\u0009";
    KeyBacktab = "\u001bOP\u0009";
    KeyUp    = "\u001b[A";
    KeyDown  = "\u001b[B";
    KeyRight  = "\u001b[C";
    KeyLeft  = "\u001b[D";
    KP0  = "\u001bOp";
    KP1  = "\u001bOq";
    KP2  = "\u001bOr";
    KP3  = "\u001bOs";
    KP4  = "\u001bOt";
    KP5  = "\u001bOu";
    KP6  = "\u001bOv";
    KP7  = "\u001bOw";
    KP8  = "\u001bOx";
    KP9  = "\u001bOy";
    KPMinus  = "\u001bOm";
    KPComma  = "\u001bOl";
    KPPeriod  = "\u001bOn";
    KPEnter  = "\u001bOM";

    /* ... */
  //}

  public Dimension getSize() {
    return new Dimension(32, 80);
  }

  public String getTerminalType() { return terminalID; }

  /**
   * Handle events for the terminal. Only accept events for the scroll
   * bar. Any other events have to be propagated to the parent.
   * @param evt the event
   */


  /**
   * Dummy method to handle input events (String).
   * This is only needed if our parent is not TerminalHost
   * @param s String to handle
   * @return always true
   * @see display.TerminalHost
   */
  public boolean send(String s) {
    //putString(s);
    return true;
  }

  private void handle_dcs(String dcs) {
    System.out.println("DCS: "+dcs);
  }
  private void handle_osc(String osc) {
    System.out.println("OSC: "+osc);
  }

 

  public final static char unimap[] = {
//#
//#    Name:     cp437_DOSLatinUS to Unicode table
//#    Unicode version: 1.1
//#    Table version: 1.1
//#    Table format:  Format A
//#    Date:          03/31/95
//#    Authors:       Michel Suignard <michelsu@microsoft.com>
//#                   Lori Hoerth <lorih@microsoft.com>
//#    General notes: none
//#
//#    Format: Three tab-separated columns
//#        Column #1 is the cp1255_WinHebrew code (in hex)
//#        Column #2 is the Unicode (in hex as 0xXXXX)
//#        Column #3 is the Unicode name (follows a comment sign, '#')
//#
//#    The entries are in cp437_DOSLatinUS order
//#

  0x0000,// #NULL
  0x0001,// #START OF HEADING
  0x0002,// #START OF TEXT
  0x0003,// #END OF TEXT
  0x0004,// #END OF TRANSMISSION
  0x0005,// #ENQUIRY
  0x0006,// #ACKNOWLEDGE
  0x0007,// #BELL
  0x0008,// #BACKSPACE
  0x0009,// #HORIZONTAL TABULATION
  0x000a,// #LINE FEED
  0x000b,// #VERTICAL TABULATION
  0x000c,// #FORM FEED
  0x000d,// #CARRIAGE RETURN
  0x000e,// #SHIFT OUT
  0x000f,// #SHIFT IN
  0x0010,// #DATA LINK ESCAPE
  0x0011,// #DEVICE CONTROL ONE
  0x0012,// #DEVICE CONTROL TWO
  0x0013,// #DEVICE CONTROL THREE
  0x0014,// #DEVICE CONTROL FOUR
  0x0015,// #NEGATIVE ACKNOWLEDGE
  0x0016,// #SYNCHRONOUS IDLE
  0x0017,// #END OF TRANSMISSION BLOCK
  0x0018,// #CANCEL
  0x0019,// #END OF MEDIUM
  0x001a,// #SUBSTITUTE
  0x001b,// #ESCAPE
  0x001c,// #FILE SEPARATOR
  0x001d,// #GROUP SEPARATOR
  0x001e,// #RECORD SEPARATOR
  0x001f,// #UNIT SEPARATOR
  0x0020,// #SPACE
  0x0021,// #EXCLAMATION MARK
  0x0022,// #QUOTATION MARK
  0x0023,// #NUMBER SIGN
  0x0024,// #DOLLAR SIGN
  0x0025,// #PERCENT SIGN
  0x0026,// #AMPERSAND
  0x0027,// #APOSTROPHE
  0x0028,// #LEFT PARENTHESIS
  0x0029,// #RIGHT PARENTHESIS
  0x002a,// #ASTERISK
  0x002b,// #PLUS SIGN
  0x002c,// #COMMA
  0x002d,// #HYPHEN-MINUS
  0x002e,// #FULL STOP
  0x002f,// #SOLIDUS
  0x0030,// #DIGIT ZERO
  0x0031,// #DIGIT ONE
  0x0032,// #DIGIT TWO
  0x0033,// #DIGIT THREE
  0x0034,// #DIGIT FOUR
  0x0035,// #DIGIT FIVE
  0x0036,// #DIGIT SIX
  0x0037,// #DIGIT SEVEN
  0x0038,// #DIGIT EIGHT
  0x0039,// #DIGIT NINE
  0x003a,// #COLON
  0x003b,// #SEMICOLON
  0x003c,// #LESS-THAN SIGN
  0x003d,// #EQUALS SIGN
  0x003e,// #GREATER-THAN SIGN
  0x003f,// #QUESTION MARK
  0x0040,// #COMMERCIAL AT
  0x0041,// #LATIN CAPITAL LETTER A
  0x0042,// #LATIN CAPITAL LETTER B
  0x0043,// #LATIN CAPITAL LETTER C
  0x0044,// #LATIN CAPITAL LETTER D
  0x0045,// #LATIN CAPITAL LETTER E
  0x0046,// #LATIN CAPITAL LETTER F
  0x0047,// #LATIN CAPITAL LETTER G
  0x0048,// #LATIN CAPITAL LETTER H
  0x0049,// #LATIN CAPITAL LETTER I
  0x004a,// #LATIN CAPITAL LETTER J
  0x004b,// #LATIN CAPITAL LETTER K
  0x004c,// #LATIN CAPITAL LETTER L
  0x004d,// #LATIN CAPITAL LETTER M
  0x004e,// #LATIN CAPITAL LETTER N
  0x004f,// #LATIN CAPITAL LETTER O
  0x0050,// #LATIN CAPITAL LETTER P
  0x0051,// #LATIN CAPITAL LETTER Q
  0x0052,// #LATIN CAPITAL LETTER R
  0x0053,// #LATIN CAPITAL LETTER S
  0x0054,// #LATIN CAPITAL LETTER T
  0x0055,// #LATIN CAPITAL LETTER U
  0x0056,// #LATIN CAPITAL LETTER V
  0x0057,// #LATIN CAPITAL LETTER W
  0x0058,// #LATIN CAPITAL LETTER X
  0x0059,// #LATIN CAPITAL LETTER Y
  0x005a,// #LATIN CAPITAL LETTER Z
  0x005b,// #LEFT SQUARE BRACKET
  0x005c,// #REVERSE SOLIDUS
  0x005d,// #RIGHT SQUARE BRACKET
  0x005e,// #CIRCUMFLEX ACCENT
  0x005f,// #LOW LINE
  0x0060,// #GRAVE ACCENT
  0x0061,// #LATIN SMALL LETTER A
  0x0062,// #LATIN SMALL LETTER B
  0x0063,// #LATIN SMALL LETTER C
  0x0064,// #LATIN SMALL LETTER D
  0x0065,// #LATIN SMALL LETTER E
  0x0066,// #LATIN SMALL LETTER F
  0x0067,// #LATIN SMALL LETTER G
  0x0068,// #LATIN SMALL LETTER H
  0x0069,// #LATIN SMALL LETTER I
  0x006a,// #LATIN SMALL LETTER J
  0x006b,// #LATIN SMALL LETTER K
  0x006c,// #LATIN SMALL LETTER L
  0x006d,// #LATIN SMALL LETTER M
  0x006e,// #LATIN SMALL LETTER N
  0x006f,// #LATIN SMALL LETTER O
  0x0070,// #LATIN SMALL LETTER P
  0x0071,// #LATIN SMALL LETTER Q
  0x0072,// #LATIN SMALL LETTER R
  0x0073,// #LATIN SMALL LETTER S
  0x0074,// #LATIN SMALL LETTER T
  0x0075,// #LATIN SMALL LETTER U
  0x0076,// #LATIN SMALL LETTER V
  0x0077,// #LATIN SMALL LETTER W
  0x0078,// #LATIN SMALL LETTER X
  0x0079,// #LATIN SMALL LETTER Y
  0x007a,// #LATIN SMALL LETTER Z
  0x007b,// #LEFT CURLY BRACKET
  0x007c,// #VERTICAL LINE
  0x007d,// #RIGHT CURLY BRACKET
  0x007e,// #TILDE
  0x007f,// #DELETE
  0x00c7,// #LATIN CAPITAL LETTER C WITH CEDILLA
  0x00fc,// #LATIN SMALL LETTER U WITH DIAERESIS
  0x00e9,// #LATIN SMALL LETTER E WITH ACUTE
  0x00e2,// #LATIN SMALL LETTER A WITH CIRCUMFLEX
  0x00e4,// #LATIN SMALL LETTER A WITH DIAERESIS
  0x00e0,// #LATIN SMALL LETTER A WITH GRAVE
  0x00e5,// #LATIN SMALL LETTER A WITH RING ABOVE
  0x00e7,// #LATIN SMALL LETTER C WITH CEDILLA
  0x00ea,// #LATIN SMALL LETTER E WITH CIRCUMFLEX
  0x00eb,// #LATIN SMALL LETTER E WITH DIAERESIS
  0x00e8,// #LATIN SMALL LETTER E WITH GRAVE
  0x00ef,// #LATIN SMALL LETTER I WITH DIAERESIS
  0x00ee,// #LATIN SMALL LETTER I WITH CIRCUMFLEX
  0x00ec,// #LATIN SMALL LETTER I WITH GRAVE
  0x00c4,// #LATIN CAPITAL LETTER A WITH DIAERESIS
  0x00c5,// #LATIN CAPITAL LETTER A WITH RING ABOVE
  0x00c9,// #LATIN CAPITAL LETTER E WITH ACUTE
  0x00e6,// #LATIN SMALL LIGATURE AE
  0x00c6,// #LATIN CAPITAL LIGATURE AE
  0x00f4,// #LATIN SMALL LETTER O WITH CIRCUMFLEX
  0x00f6,// #LATIN SMALL LETTER O WITH DIAERESIS
  0x00f2,// #LATIN SMALL LETTER O WITH GRAVE
  0x00fb,// #LATIN SMALL LETTER U WITH CIRCUMFLEX
  0x00f9,// #LATIN SMALL LETTER U WITH GRAVE
  0x00ff,// #LATIN SMALL LETTER Y WITH DIAERESIS
  0x00d6,// #LATIN CAPITAL LETTER O WITH DIAERESIS
  0x00dc,// #LATIN CAPITAL LETTER U WITH DIAERESIS
  0x00a2,// #CENT SIGN
  0x00a3,// #POUND SIGN
  0x00a5,// #YEN SIGN
  0x20a7,// #PESETA SIGN
  0x0192,// #LATIN SMALL LETTER F WITH HOOK
  0x00e1,// #LATIN SMALL LETTER A WITH ACUTE
  0x00ed,// #LATIN SMALL LETTER I WITH ACUTE
  0x00f3,// #LATIN SMALL LETTER O WITH ACUTE
  0x00fa,// #LATIN SMALL LETTER U WITH ACUTE
  0x00f1,// #LATIN SMALL LETTER N WITH TILDE
  0x00d1,// #LATIN CAPITAL LETTER N WITH TILDE
  0x00aa,// #FEMININE ORDINAL INDICATOR
  0x00ba,// #MASCULINE ORDINAL INDICATOR
  0x00bf,// #INVERTED QUESTION MARK
  0x2310,// #REVERSED NOT SIGN
  0x00ac,// #NOT SIGN
  0x00bd,// #VULGAR FRACTION ONE HALF
  0x00bc,// #VULGAR FRACTION ONE QUARTER
  0x00a1,// #INVERTED EXCLAMATION MARK
  0x00ab,// #LEFT-POINTING DOUBLE ANGLE QUOTATION MARK
  0x00bb,// #RIGHT-POINTING DOUBLE ANGLE QUOTATION MARK
  0x2591,// #LIGHT SHADE
  0x2592,// #MEDIUM SHADE
  0x2593,// #DARK SHADE
  0x2502,// #BOX DRAWINGS LIGHT VERTICAL
  0x2524,// #BOX DRAWINGS LIGHT VERTICAL AND LEFT
  0x2561,// #BOX DRAWINGS VERTICAL SINGLE AND LEFT DOUBLE
  0x2562,// #BOX DRAWINGS VERTICAL DOUBLE AND LEFT SINGLE
  0x2556,// #BOX DRAWINGS DOWN DOUBLE AND LEFT SINGLE
  0x2555,// #BOX DRAWINGS DOWN SINGLE AND LEFT DOUBLE
  0x2563,// #BOX DRAWINGS DOUBLE VERTICAL AND LEFT
  0x2551,// #BOX DRAWINGS DOUBLE VERTICAL
  0x2557,// #BOX DRAWINGS DOUBLE DOWN AND LEFT
  0x255d,// #BOX DRAWINGS DOUBLE UP AND LEFT
  0x255c,// #BOX DRAWINGS UP DOUBLE AND LEFT SINGLE
  0x255b,// #BOX DRAWINGS UP SINGLE AND LEFT DOUBLE
  0x2510,// #BOX DRAWINGS LIGHT DOWN AND LEFT
  0x2514,// #BOX DRAWINGS LIGHT UP AND RIGHT
  0x2534,// #BOX DRAWINGS LIGHT UP AND HORIZONTAL
  0x252c,// #BOX DRAWINGS LIGHT DOWN AND HORIZONTAL
  0x251c,// #BOX DRAWINGS LIGHT VERTICAL AND RIGHT
  0x2500,// #BOX DRAWINGS LIGHT HORIZONTAL
  0x253c,// #BOX DRAWINGS LIGHT VERTICAL AND HORIZONTAL
  0x255e,// #BOX DRAWINGS VERTICAL SINGLE AND RIGHT DOUBLE
  0x255f,// #BOX DRAWINGS VERTICAL DOUBLE AND RIGHT SINGLE
  0x255a,// #BOX DRAWINGS DOUBLE UP AND RIGHT
  0x2554,// #BOX DRAWINGS DOUBLE DOWN AND RIGHT
  0x2569,// #BOX DRAWINGS DOUBLE UP AND HORIZONTAL
  0x2566,// #BOX DRAWINGS DOUBLE DOWN AND HORIZONTAL
  0x2560,// #BOX DRAWINGS DOUBLE VERTICAL AND RIGHT
  0x2550,// #BOX DRAWINGS DOUBLE HORIZONTAL
  0x256c,// #BOX DRAWINGS DOUBLE VERTICAL AND HORIZONTAL
  0x2567,// #BOX DRAWINGS UP SINGLE AND HORIZONTAL DOUBLE
  0x2568,// #BOX DRAWINGS UP DOUBLE AND HORIZONTAL SINGLE
  0x2564,// #BOX DRAWINGS DOWN SINGLE AND HORIZONTAL DOUBLE
  0x2565,// #BOX DRAWINGS DOWN DOUBLE AND HORIZONTAL SINGLE
  0x2559,// #BOX DRAWINGS UP DOUBLE AND RIGHT SINGLE
  0x2558,// #BOX DRAWINGS UP SINGLE AND RIGHT DOUBLE
  0x2552,// #BOX DRAWINGS DOWN SINGLE AND RIGHT DOUBLE
  0x2553,// #BOX DRAWINGS DOWN DOUBLE AND RIGHT SINGLE
  0x256b,// #BOX DRAWINGS VERTICAL DOUBLE AND HORIZONTAL SINGLE
  0x256a,// #BOX DRAWINGS VERTICAL SINGLE AND HORIZONTAL DOUBLE
  0x2518,// #BOX DRAWINGS LIGHT UP AND LEFT
  0x250c,// #BOX DRAWINGS LIGHT DOWN AND RIGHT
  0x2588,// #FULL BLOCK
  0x2584,// #LOWER HALF BLOCK
  0x258c,// #LEFT HALF BLOCK
  0x2590,// #RIGHT HALF BLOCK
  0x2580,// #UPPER HALF BLOCK
  0x03b1,// #GREEK SMALL LETTER ALPHA
  0x00df,// #LATIN SMALL LETTER SHARP S
  0x0393,// #GREEK CAPITAL LETTER GAMMA
  0x03c0,// #GREEK SMALL LETTER PI
  0x03a3,// #GREEK CAPITAL LETTER SIGMA
  0x03c3,// #GREEK SMALL LETTER SIGMA
  0x00b5,// #MICRO SIGN
  0x03c4,// #GREEK SMALL LETTER TAU
  0x03a6,// #GREEK CAPITAL LETTER PHI
  0x0398,// #GREEK CAPITAL LETTER THETA
  0x03a9,// #GREEK CAPITAL LETTER OMEGA
  0x03b4,// #GREEK SMALL LETTER DELTA
  0x221e,// #INFINITY
  0x03c6,// #GREEK SMALL LETTER PHI
  0x03b5,// #GREEK SMALL LETTER EPSILON
  0x2229,// #INTERSECTION
  0x2261,// #IDENTICAL TO
  0x00b1,// #PLUS-MINUS SIGN
  0x2265,// #GREATER-THAN OR EQUAL TO
  0x2264,// #LESS-THAN OR EQUAL TO
  0x2320,// #TOP HALF INTEGRAL
  0x2321,// #BOTTOM HALF INTEGRAL
  0x00f7,// #DIVISION SIGN
  0x2248,// #ALMOST EQUAL TO
  0x00b0,// #DEGREE SIGN
  0x2219,// #BULLET OPERATOR
  0x00b7,// #MIDDLE DOT
  0x221a,// #SQUARE ROOT
  0x207f,// #SUPERSCRIPT LATIN SMALL LETTER N
  0x00b2,// #SUPERSCRIPT TWO
  0x25a0,// #BLACK SQUARE
  0x00a0,// #NO-BREAK SPACE
  };

  public char map_cp850_unicode(char x) {
    if (x>=0x100)
      return x;
    return unimap[x];
  }

  
 public void putString(String s)
{}

}
