import java.util.Properties;
import java.io.*;
import org.gxos.*;
import org.gxos.schema.*;
import org.gxos.schema.types.*;
import org.gxos.gndi.context.*;
import org.gxos.debug.Log;

/**
 * Example for deleting objects and destroying contexts.
 * This example undoes what is done in Example1.java.
 * @author Ozgur Balsoy
 * @version 1.0
 */

 public class Example3 {

  public Example3(Properties env) {

	try {

            // With the given config values (either from the environment,
            // or configuration file) construct the initial context.
            // See main() first.
            //
            // Contexts in JNDI are named containers which can contain
            // other named contexts and objects. For each start, we need
            // a startup container which is the initial context. This is
            // constructed in ContextManager. To get that context, use
            // getContext method.

            System.out.println("Initializing environment...");
            ContextManager.initialize(env);
            GxContext rootctx = ContextManager.getContext();

            // for all deletions, we will use rootctx only.
            // first we need to remove bound objects.

            rootctx.unbind("myroot/users/balsoy");
            System.out.println("Removed //myroot/users/balsoy");

            rootctx.unbind("myroot/users/jdoe");
            System.out.println("Removed //myroot/users/jdoe");

            rootctx.unbind("myroot/devices/localhost");
            System.out.println("Removed //myroot/users/localhost");

            // other methods which could be used:
            /*
                GxContext users = (GxContext) rootctx.lookup("myroot/users");
                users.unbind("balsoy");
                users.unbind("jdoe");
                GxContext devices = (GxContext) rootctx.lookup("myroot/devices");
                device.unbind("dirac");
            */

            // remove contexts

            rootctx.destroySubcontext("myroot/devices");
            System.out.println("Removed //myroot/devices");

            rootctx.destroySubcontext("myroot/users");
            System.out.println("Removed //myroot/users");

            // finally...
            rootctx.destroySubcontext("myroot");
            System.out.println("Removed //myroot");

	} catch (Exception e) {
	    e.printStackTrace();
	}
    }

  public static void main(String[] args) {

    if(args.length > 0 && args[0].equals("-d"))
        Log.debug(true);

    try {
        String configFile = (String)System.getProperties().get(Defaults.GXOS_CONFIG);
        Properties env = Defaults.getConfiguration(configFile);

        Example3 example = new Example3(env);

    } catch(Exception e) {
        e.printStackTrace();
    }
  }
}