/***********************************************************************
 * Program: GraphicsManager.java
 * @author Marlon Pierce
 *
 * Manages the graphics objects.  A given scene should be composed of 
 * various graphics pieces (specifically, children of the SVGShape 
 * abstract class).  The GraphicsManager class creates all of these
 * constituent classes and provides the data needed in the specific 
 * application.
 ***********************************************************************/

//import org.apache.batik.apps.svgviewer.Application;
//import org.apache.batik.apps.svgviewer.ViewerFrame;
import org.apache.batik.util.awt.svg.SVGGraphics2D;

import java.io.*;
import java.util.*;


public class GraphicsManager {

    Main main;
    SVGChemistry svgChem;
    Vector childList;
    int xpart,ypart;

    public GraphicsManager(Main main) {
	this.main=main;
	childList=new Vector();
    }
    
    /**
     * Creates a vector of all of the WhiteBoardComp objects, so
     * that we can recover them later.
     */
    public void setUpCompList(){
    	System.out.println("Setting up list of components");
	svgChem=new SVGChemistry(main);
	xpart=svgChem.getXPart();
	ypart=svgChem.getYPart();
	childList.add(svgChem);
	System.out.println("Comp list set up");
    }

    /**
     * Returns the WhiteBoardComp object at the given index from the
     * childList Vector.
     */
    public WhiteBoardComp getWBShape(int index){
	return (WhiteBoardComp)childList.get(index);
    }
    
    /**
     * Steps through the childList vector and invokes each one's loadData()
     * method.  The data may be partitioned into equally sized regions for
     * faster display.  If not, xpart=1 and ypart=1.
     */
    public void loadDataFiles(String[] args){
	System.out.println(args.length+" "+childList.size());
	for(int i=0;i<args.length;i++){
	    ((WhiteBoardComp)childList.get(i)).loadDataFile(args[i]);
	}
    }
    
    /**
     * Steps through the childList vector and invokes each one's init()
     * method.
     */
    public void initChildren() {
	setUpCompList();
	System.out.println("Sweep through child list");
	for(int i=0;i<childList.size();i++) {
	    System.out.println(i);
	    ((WhiteBoardComp)childList.get(i)).init();
	}
    }
    
    /**
     * Steps through the childList vector and invokes each one's 
     * drawChildren method.
     */    
    public void drawChildren(SVGGraphics2D svgGenerator) {
	for(int i=0;i<childList.size();i++) {
	    for(int ipart=0;ipart<xpart;ipart++) {
  		for(int jpart=0;jpart<ypart;jpart++) {
  		    ((WhiteBoardComp)childList.get(i)).
  			drawShape(svgGenerator,ipart,jpart);
  		}
  	    }
  	}
    }
    
    public void test() {
	System.out.println("GraphicsManager test successful.");
    }
}


