import javax.naming.*;
import java.util.Properties;
import java.io.*;
import org.gxos.debug.Log;
import org.gxos.*;
import org.gxos.schema.User;
import org.gxos.gndi.context.*;
import com.anabas.vc.schema.*;

/**
 * Example for creating a root context.
 * @author Ozgur Balsoy
 * @version 1.0
 */

public class CreateUser2 {

  public CreateUser2(Properties env) {

	try {
            // free style root construction
            ContextManager.initialize(env);
            GxContext topctx = ContextManager.getContext();
            GxContext root = (GxContext)topctx.createSubcontext("mydb");

            root.createSubcontext("users");
            root.createSubcontext("meetings");

            User u = (User)Users.createUser(new User(), "balsoy", "pwd", "Ozgur Balsoy", "", "", "ozgur@foo.bar.com");
            Person p = u.getVCPersonalInfo().getPerson();
            Address a = new Address();
            a.setAddresstype("work");
            a.setStreet("Florida State University");
            a.setExtension("400 Dirac Science Lib");
            a.setCity("Tallahassee");
            a.setState("FL");
            a.setZip("32306");
            p.getContact().addAddress(a);

            a = new Address();
            a.setAddresstype("home");
            a.setStreet("101 First Street");
            a.setCity("Tallahassee");
            a.setState("FL");
            a.setZip("32000");
            p.getContact().addAddress(a);

            root.bind("users/balsoy", u);

            u = (User)Users.createUser(new User(), "jdoe", "pwd", "Jane Doe", "", "", "jane@fsu.edu");
            p = u.getVCPersonalInfo().getPerson();

            Phone phone = new Phone();
            phone.setPhonetype("someotherstuff");
            phone.setPArea("850");
            phone.setPNumber("5551122");
            p.getContact().addPhone(phone);

            root.bind("users/jdoe", u);

            // list users
            PrintWriter pw = new PrintWriter(System.out, true /*autoflush*/);

            // object lookup
            u = (User)root.lookup("users/balsoy");
            // print xml
            Defaults.marshal(u, pw);
            pw.println("\n\n");
            // object lookup
            u = (User)root.lookup("users/jdoe");
            // print xml
            Defaults.marshal(u, pw);

            NamingEnumeration users = root.listBindings("users");
            while(users.hasMore()) {
                Binding b = (Binding)users.next();
                u = (User)b.getObject();
                pw.println("\nUser :"+u.getNodeName()); // UserID is copied to NodeName
                String n = u.getVCPersonalInfo().getPerson().getName().getFirstName();
                n += " " + u.getVCPersonalInfo().getPerson().getName().getMidName();
                n += " " + u.getVCPersonalInfo().getPerson().getName().getLastName();
                pw.println("\nName :"+ n);
            }

	} catch (Exception e) {
	    e.printStackTrace();
	}
    }

  public static void main(String[] args) {

    try {
        String configFile = (String)System.getProperties().get(Defaults.GXOS_CONFIG);
        Properties env = Defaults.getConfiguration(configFile);
        Log.debug(false);
        CreateUser2 cr = new CreateUser2(env);
    } catch(Exception e) {
        e.printStackTrace();
    }
  }
}