import java.util.Properties;
import javax.naming.*;
import java.io.*;
import org.gxos.*;
import org.gxos.schema.*;
import org.gxos.schema.types.*;
import org.gxos.gndi.context.*;
import org.gxos.debug.Log;
import org.exolab.castor.xml.MarshalException;
import org.exolab.castor.xml.ValidationException;

/**
 * Example for listing objects.
 * @author Ozgur Balsoy
 * @version 1.0
 */

 public class Example2 {

  public Example2(Properties env) {

	try {
            // needed for marshalling objects' content to System.out.
            PrintWriter systemWriter = new PrintWriter(System.out);

            // With the given config values (either from the environment,
            // or configuration file) construct the initial context.
            // See main() first.
            //
            // Contexts in JNDI are named containers which can contain
            // other named contexts and objects. For each start, we need
            // a startup container which is the initial context. This is
            // constructed in ContextManager. To get that context, use
            // getContext method.

            ContextManager.initialize(env);
            GxContext rootctx = ContextManager.getContext();

            // Objects or subcontexts in any context can be listed in two ways:
            // Listing names with class types, and names with bound objects.
            // list(name) and listBindings(name) serve this purpose respectively.

            // Let's list object names and their class names first.
            // This example assumes that you have run the Example1.java first.
            // list() method returns an enumeration of NameClassPair objects,
            // each holding a name and a class name.

            systemWriter.println("\nListing objects in myroot");
            NamingEnumeration ne = rootctx.list("myroot");
            while(ne.hasMore()) {
                NameClassPair naming = (NameClassPair)ne.next();
                systemWriter.println("Object bound to the name "+naming.getName());
                systemWriter.println(" is an instance of the class "+naming.getClassName());
            }

            // Now, let's list each user object and its content. listBindings()
            // method returns an enumeration of Binding objects, each holding
            // a name and object.

            systemWriter.println("\nListing objects in myroot/users");
            NamingEnumeration be = rootctx.listBindings("myroot/users");
            while(be.hasMore()) {
                Binding binding = (Binding)be.next();
                systemWriter.println("\nObject bound to the name "+binding.getName());
                Object object = binding.getObject();
                if(object instanceof Context) {

                    systemWriter.println(" is a context and also a GXOS collection.");
                    // if this is a context, let's retrive its XML content.

                    // These commented lines are clearer version of the line below.
                    // GxContext gx = (GxContext)object;
                    // Object obj = gx.lookup(GxContext.SELF);
                    // Collection coll = (Collection)obj;
                    Collection coll = (Collection)((GxContext)object).lookup(GxContext.SELF);
                    systemWriter.println("\nObject marhsalled:");
                    Defaults.marshal(coll, systemWriter);

                } else if (object instanceof TreeObject) {

                    systemWriter.println(" is a GXOS object.\nObject marshalled:");
                    Defaults.marshal((TreeObject)object, systemWriter);

                }
            }

            System.out.println("\nTry Example3 to remove these objects.");

	} catch (Exception e) {
	    e.printStackTrace();
	}
    }

  public static void main(String[] args) {

    if(args.length > 0 && args[0].equals("-d"))
        Log.debug(true);

    try {
        String configFile = (String)System.getProperties().get(Defaults.GXOS_CONFIG);
        Properties env = Defaults.getConfiguration(configFile);

        Example2 example = new Example2(env);

    } catch(Exception e) {
        e.printStackTrace();
    }
  }
}