import java.util.Properties;
import java.io.*;
import org.gxos.*;
import org.gxos.schema.*;
import org.gxos.schema.types.*;
import org.gxos.gndi.context.*;
import org.gxos.debug.Log;

/**
 * Example for renaming objects and contexts
 *
 * @author Ozgur Balsoy
 * @version 1.0
 */

 public class Example4 {

  public Example4(Properties env) {

	try {

            // With the given config values (either from the environment,
            // or configuration file) construct the initial context.
            // See main() first.
            //
            // Contexts in JNDI are named containers which can contain
            // other named contexts and objects. For each start, we need
            // a startup container which is the initial context. This is
            // constructed in ContextManager. Using getRootContext method
            // users of this API set their working environment.

            ContextManager.initialize(env);
            GxContext rootctx = ContextManager.getContext();

            // Using rename(oldname, newname) method, objects can be renamed
            // as in the next line. However, at this version, context renaming
            // is NOT supported.

            rootctx.rename("myroot/users/balsoy","myroot/users/ozgur");

	} catch (Exception e) {
	    e.printStackTrace();
	}
    }

  public static void main(String[] args) {

    if(args.length > 0 && args[0].equals("-d"))
        Log.debug(true);

    try {
        String configFile = (String)System.getProperties().get(Defaults.GXOS_CONFIG);
        Properties env = Defaults.getConfiguration(configFile);

        Example4 example = new Example4(env);

    } catch(Exception e) {
        e.printStackTrace();
    }
  }
}